from __future__ import annotations
import lightningchart
from lightningchart import conf, Themes
from lightningchart.charts import GeneralMethods, TitleMethods, Chart
from lightningchart.instance import Instance


class MapChart(GeneralMethods, TitleMethods):

    def __init__(
            self,
            map_type: str = 'World',
            theme: Themes = Themes.White,
            title: str = None,
            license: str = None
    ):
        """Chart class for visualizing a Map of a selected part of the world. Defaults to the entire world.

        Args:
            map_type (str): "Africa" | "Asia" | "Australia" | "Canada" | "Europe" |
                "NorthAmerica" | "SouthAmerica" | "USA" | "World"
            theme (Themes): Overall theme of the chart.
            title (str): Title of the chart.
            license (str): License key.
        """
        map_types = (
            "Africa", "Asia", "Australia", "Canada", "Europe",
            "NorthAmerica", "SouthAmerica", "USA", "World"
        )
        if map_type not in map_types:
            raise ValueError(
                f"Expected map_type to be one of {map_types}"
                f", but got '{map_type}'."
            )

        instance = Instance()
        Chart.__init__(self, instance)
        self.instance.send(self.id, 'mapChart', {
            'mapType': map_type,
            'theme': theme.value,
            'license': license or conf.LICENSE_KEY
        })
        if title:
            self.set_title(title)

    def invalidate_region_values(self, region_values: list[dict]):
        """Invalidate numeric values associated with each region of the Map.

        Args:
            region_values (list[dict]): List of {"ISO_A3": string, "value": number} dictionaries.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'invalidateRegionValues', {'values': region_values})
        return self

    def set_palette_colors(
            self,
            steps: list[dict[str, any]],
            look_up_property: str = 'value',
            interpolate: bool = True,
            percentage_values: bool = False
    ):
        """Define a palette for dynamically looked up fill coloring for regions.

        Args:
            steps (list[dict]): List of {"value": number, "color": Color} dictionaries.
            interpolate (bool): Enables automatic linear interpolation between color steps.
            look_up_property (str): "value" | "x" | "y" | "z"
            percentage_values (bool): Whether values represent percentages or explicit values.

        Returns:
            The instance of the class for fluent interface.
        """
        for i in steps:
            if 'color' in i and type(i.get('color')) != 'string':
                i['color'] = i['color'].get_hex()

        self.instance.send(self.id, 'setPalettedFillStyle', {
            'steps': steps,
            'lookUpProperty': look_up_property,
            'interpolate': interpolate,
            'percentageValues': percentage_values
        })
        return self

    def set_min_max_palette_colors(
            self,
            min_value: int | float,
            max_value: int | float,
            min_color: lightningchart.Color,
            max_color: lightningchart.Color,
            look_up_property: str = 'value',
            percentage_values: bool = False
    ):
        """Define a palette for dynamically looked up fill coloring regions.

        Args:
            min_value (int | float): Minimum possible value for any given datapoint.
            max_value (int | float): Maximum possible value for any given datapoint.
            min_color (Color): Color of the minimum value.
            max_color (Color): Color of the maximum value.
            look_up_property (str): "value" | "x" | "y" | "z"
            percentage_values (bool): Whether values represent percentages or explicit values.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setPalettedFillStyleMinMax', {
            'minValue': min_value,
            'maxValue': max_value,
            'minColor': min_color.get_hex(),
            'maxColor': max_color.get_hex(),
            'lookUpProperty': look_up_property,
            'percentageValues': percentage_values
        })
        return self

    def set_solid_color(self, color: lightningchart.Color):
        """All Map regions are filled with a single color.

        Args:
            color (Color): Color for all regions.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setSolidFillStyle', {'color': color.get_hex()})
        return self

    def set_highlight_on_hover(self, enabled: bool):
        """Set highlight on mouse hover enabled or disabled.

        Args:
            enabled (bool): Boolean flag.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setHighlightOnHover', {'enabled': enabled})
        return self

    def set_outlier_region_color(self, color: lightningchart.Color):
        """Set color of outlier regions (parts of map that are visible, but not interactable with active map type).

        Args:
            color (Color): Color of outlier regions.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setOutlierRegionFillStyle', {'color': color.get_hex()})
        return self

    def set_outlier_region_stroke(self, thickness: int | float, color: lightningchart.Color):
        """Set stroke of outlier regions (parts of map that are visible, but not interactable with active map type).

        Args:
            thickness (int | float): Thickness of the stroke.
            color (Color): Color of the stroke.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setOutlierRegionStrokeStyle', {'thickness': thickness, 'color': color.get_hex()})
        return self

    def set_separate_region_color(self, color: lightningchart.Color):
        """Set color of separate regions, which are visual components surrounding areas such as Alaska and Hawaii.

        Args:
            color (Color): Color of separate regions.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setSeparateRegionFillStyle', {'color': color.get_hex()})
        return self

    def set_separate_region_stroke(self, thickness: int | float, color: lightningchart.Color):
        """Set stroke of Separate regions, which are visual components surrounding areas such as Alaska and Hawaii.

        Args:
            thickness (int | float): Thickness of the stroke.
            color (Color): Color of the stroke.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setSeparateRegionStrokeStyle', {'thickness': thickness, 'color': color.get_hex()})
        return self

    def set_stroke(self, thickness: int | float, color: lightningchart.Color):
        """Set Stroke style of Map regions.

        Args:
            thickness (int | float): Thickness of the stroke.
            color (Color): Color of the stroke.

        Returns:
            The instance of the class for fluent interface.
        """
        self.instance.send(self.id, 'setStrokeStyle', {'thickness': thickness, 'color': color.get_hex()})
        return self


class MapChartDashboard(MapChart):

    def __init__(
            self,
            instance: Instance,
            dashboard_id: str,
            column: int,
            row: int,
            colspan: int,
            rowspan: int):
        Chart.__init__(self, instance)
        self.instance.send(self.id, 'createMapChart', {
            'db': dashboard_id,
            'column': column,
            'row': row,
            'colspan': colspan,
            'rowspan': rowspan
        })
